@@ -667,11 +667,11 @@ bool LotusGraph::readZoneBegin(std::shared_ptr<WPSStream> stream, long endPos)
 
 		return true;
 	}
-	m_state->m_actualSheetId=(int) libwps::readU8(input);
+	m_state->m_actualSheetId=int(libwps::readU8(input));
 	f << "sheet[id]=" << m_state->m_actualSheetId << ",";
 	for (int i=0; i<3; ++i)   // f0=1
 	{
-		auto val=(int) libwps::readU8(input);
+		auto val=int(libwps::readU8(input));
 		if (val)
 			f << "f" << i << "=" << val << ",";
 	}
@@ -724,43 +724,43 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 		ascFile.addNote(f.str().c_str());
 		return true;
 	}
-	zone->m_ordering=(int) libwps::readU8(input);
+	zone->m_ordering=int(libwps::readU8(input));
 	for (int i=0; i<4; ++i)   // always 0?
 	{
-		auto val=(int) libwps::read8(input);
+		auto val=int(libwps::read8(input));
 		if (val)
 			f << "f" << i << "=" << val << ",";
 	}
 	int dim[4];
 	for (int i=0; i<4; ++i)   // dim3[high]=0|100
 	{
-		dim[i]=(int) libwps::read16(input);
+		dim[i]=int(libwps::read16(input));
 		if (i==3)
 			break;
-		auto val=(int) libwps::read16(input);
+		auto val=int(libwps::read16(input));
 		if (val) f << "dim" << i << "[high]=" << std::hex << val << std::dec << ",";
 	}
 	zone->m_box=WPSBox2i(Vec2i(dim[1],dim[0]),Vec2i(dim[3],dim[2]));
-	auto val=(int) libwps::read8(input);
+	auto val=int(libwps::read8(input));
 	if (val) // always 0
 		f << "f4=" << val << ",";
 	int fl;
 	switch (zone->m_type)
 	{
 	case LotusGraphInternal::ZoneMac::Line:
-		val=(int) libwps::readU8(input);
-		fl=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
+		fl=int(libwps::readU8(input));
 		if (val)
 		{
 			if (fl!=0x10)
 				f << "#line[fl]=" << std::hex << fl << std::dec << ",";
 			zone->m_lineId=val;
 		}
-		val=(int) libwps::readU8(input); // always 1?
+		val=int(libwps::readU8(input)); // always 1?
 		if (val!=1)
 			f << "g0=" << val << ",";
 		// the arrows &1 means end, &2 means begin
-		zone->m_values[0]=(int) libwps::readU8(input);
+		zone->m_values[0]=int(libwps::readU8(input));
 		if (sz<26)
 		{
 			WPS_DEBUG_MSG(("LotusGraph::readZoneData: the line zone seems too short\n"));
@@ -769,16 +769,16 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 		}
 		for (int i=0; i<2; ++i)   // always g1=0, g2=3 ?
 		{
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val!=3*i)
 				f << "g" << i+1 << "=" << val << ",";
 		}
 		break;
 	case LotusGraphInternal::ZoneMac::Rect:
-		val=(int) libwps::readU8(input); // always 1?
+		val=int(libwps::readU8(input)); // always 1?
 		if (val!=1)
 			f << "g0=" << val << ",";
-		zone->m_subType=(int) libwps::readU8(input);
+		zone->m_subType=int(libwps::readU8(input));
 		if (sz<28)
 		{
 			WPS_DEBUG_MSG(("LotusGraph::readZoneData: the rect zone seems too short\n"));
@@ -787,8 +787,8 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 		}
 		for (int i=0; i<2; ++i)
 		{
-			val=(int) libwps::readU8(input);
-			fl=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
+			fl=int(libwps::readU8(input));
 			if (!val) continue;
 			if (i==0)
 			{
@@ -801,18 +801,18 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 				f << "#surface[fl]=" << std::hex << fl << std::dec << ",";
 			zone->m_surfaceId=val;
 		}
-		val=(int) libwps::read16(input); // always 3?
+		val=int(libwps::read16(input)); // always 3?
 		if (val!=3)
 			f << "g1=" << val << ",";
 		break;
 	case LotusGraphInternal::ZoneMac::Frame:
-		val=(int) libwps::readU8(input); // always 1?
+		val=int(libwps::readU8(input)); // always 1?
 		if (val!=1)
 			f << "g0=" << val << ",";
 		// small value 1-4
-		zone->m_subType=(int) libwps::readU8(input);
-		val=(int) libwps::readU8(input);
-		fl=(int) libwps::readU8(input);
+		zone->m_subType=int(libwps::readU8(input));
+		val=int(libwps::readU8(input));
+		fl=int(libwps::readU8(input));
 		if (!val) break;;
 		if (fl!=0x40)
 			f << "#graphic[fl]=" << std::hex << fl << std::dec << ",";
@@ -820,13 +820,13 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 		// can be followed by 000000000100 : some way to determine the content ?
 		break;
 	case LotusGraphInternal::ZoneMac::Arc:
-		val=(int) libwps::readU8(input); // always 1?
+		val=int(libwps::readU8(input)); // always 1?
 		if (val!=1)
 			f << "g0=" << val << ",";
 		// always 3
-		zone->m_subType=(int) libwps::readU8(input);
-		val=(int) libwps::readU8(input);
-		fl=(int) libwps::readU8(input);
+		zone->m_subType=int(libwps::readU8(input));
+		val=int(libwps::readU8(input));
+		fl=int(libwps::readU8(input));
 		if (val)
 		{
 			if (fl!=0x10)
@@ -839,7 +839,7 @@ bool LotusGraph::readZoneData(std::shared_ptr<WPSStream> stream, long endPos, in
 			f << "###sz,";
 			break;
 		}
-		val=(int) libwps::read16(input); // always 0? maybe the angle
+		val=int(libwps::read16(input)); // always 0? maybe the angle
 		if (val)
 			f << "g1=" << val << ",";
 		break;
@@ -922,27 +922,27 @@ bool LotusGraph::readPictureDefinition(std::shared_ptr<WPSStream> stream, long e
 	{
 		WPS_DEBUG_MSG(("LotusGraph::readPictureDefinition: Oops can not find the parent frame\n"));
 	}
-	auto val=(int) libwps::readU8(input); // always 0?
+	auto val=int(libwps::readU8(input)); // always 0?
 	if (val)
 		f << "f0=" << val << ",";
 	int dim[2];
-	dim[0]=(int) libwps::readU16(input);
+	dim[0]=int(libwps::readU16(input));
 	for (int i=0; i<2; ++i)
 	{
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val)
 			f << "f" << i+1 << "=" << val << ",";
 	}
-	dim[1]=(int) libwps::readU16(input);
+	dim[1]=int(libwps::readU16(input));
 	f << "dim=" << Vec2i(dim[0], dim[1]) << ",";
-	val=(int) libwps::readU8(input);
+	val=int(libwps::readU8(input));
 	if (val)
 		f << "f3=" << val << ",";
-	auto pictSz=(int) libwps::readU16(input); // maybe 32bits
+	auto pictSz=int(libwps::readU16(input)); // maybe 32bits
 	f << "pict[sz]=" << std::hex << pictSz << std::dec << ",";
 	for (int i=0; i<3; ++i)   // always 0,0,1
 	{
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val)
 			f << "g" << i << "=" << val << ",";
 	}
@@ -969,7 +969,7 @@ bool LotusGraph::readPictureData(std::shared_ptr<WPSStream> stream, long endPos)
 		ascFile.addNote(f.str().c_str());
 		return true;
 	}
-	auto val=(int) libwps::readU8(input); // always 1?
+	auto val=int(libwps::readU8(input)); // always 1?
 	if (val!=1)
 		f << "type?=" << val << ",";
 	if (!m_state->m_actualZoneMac || m_state->m_actualZoneMac->m_type != LotusGraphInternal::ZoneMac::Frame)
@@ -985,7 +985,7 @@ bool LotusGraph::readPictureData(std::shared_ptr<WPSStream> stream, long endPos)
 #ifdef DEBUG_WITH_FILES
 	ascFile.skipZone(input->tell(), endPos-1);
 	librevenge::RVNGBinaryData data;
-	if (!libwps::readData(input, (unsigned long)(endPos-input->tell()), data))
+	if (!libwps::readData(input, static_cast<unsigned long>(endPos-input->tell()), data))
 		f << "###";
 	else
 	{
@@ -1014,7 +1014,7 @@ void LotusGraph::sendPicture(LotusGraphInternal::ZoneMac const &zone)
 	RVNGInputStreamPtr input=zone.m_stream->m_input;
 	librevenge::RVNGBinaryData data;
 	input->seek(zone.m_pictureEntry.begin(), librevenge::RVNG_SEEK_SET);
-	if (!libwps::readData(input, (unsigned long)(zone.m_pictureEntry.length()), data))
+	if (!libwps::readData(input, static_cast<unsigned long>(zone.m_pictureEntry.length()), data))
 	{
 		WPS_DEBUG_MSG(("LotusGraph::sendPicture: I can not find the picture\n"));
 		return;
@@ -1043,18 +1043,24 @@ void LotusGraph::sendTextBox(std::shared_ptr<WPSStream> stream, WPSEntry const &
 	long sz=entry.length();
 	f << "GraphTextBox[data]:";
 	input->seek(pos, librevenge::RVNG_SEEK_SET);
-	auto val=(int) libwps::readU8(input); // always 1?
+	auto val=int(libwps::readU8(input)); // always 1?
 	if (val!=1) f << "f0=" << val << ",";
 	auto fontType = m_mainParser.getDefaultFontType();
 	WPSFont font=WPSFont::getDefault();
 	m_listener->setFont(font);
 	bool actualFlags[7]= {false, false, false, false, false, false, false };
-	for (long i=1; i<sz; ++i)
+	std::string text;
+	for (long i=1; i<=sz; ++i)
 	{
-		auto c=(char) libwps::readU8(input);
+		auto c=i+1==sz ? '\0' : char(libwps::readU8(input));
+		if ((c==0 || c==0xe || c==0xf) && !text.empty())
+		{
+			m_listener->insertUnicodeString(libwps_tools_win::Font::unicodeString(text, fontType));
+			text.clear();
+		}
 		if (c==0)
 		{
-			if (i+2<sz)
+			if (int(i+2)<int(sz)) // g++ v7 warms if we test i+2<sz :-~
 			{
 				WPS_DEBUG_MSG(("LotusGraph::sendTextBox: find a 0 char\n"));
 				f << "[###0]";
@@ -1064,7 +1070,7 @@ void LotusGraph::sendTextBox(std::shared_ptr<WPSStream> stream, WPSEntry const &
 		if (c!=0xe && c!=0xf)
 		{
 			f << c;
-			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode((unsigned char)c,fontType));
+			text.push_back(c);
 			continue;
 		}
 		if (i+1>=sz)
@@ -1072,7 +1078,7 @@ void LotusGraph::sendTextBox(std::shared_ptr<WPSStream> stream, WPSEntry const &
 			WPS_DEBUG_MSG(("LotusGraph::sendTextBox: find modifier in last pos\n"));
 			f << "[###" << int(c) << "]";
 		}
-		auto mod=(int) libwps::readU8(input);
+		auto mod=int(libwps::readU8(input));
 		++i;
 		if (c==0xf)
 		{
@@ -1100,8 +1106,8 @@ void LotusGraph::sendTextBox(std::shared_ptr<WPSStream> stream, WPSEntry const &
 		bool ok=true;
 		for (int d=0; d<szParam; ++d)
 		{
-			auto mod1=(int) libwps::readU8(input);
-			val=(int) libwps::readU8(input);
+			auto mod1=int(libwps::readU8(input));
+			val=int(libwps::readU8(input));
 			static int const decal[]= {1,0,3,2};
 			if (mod1==0xe && (val>='0'&&val<='9'))
 			{
@@ -1134,13 +1140,13 @@ void LotusGraph::sendTextBox(std::shared_ptr<WPSStream> stream, WPSEntry const &
 		case 7:
 		{
 			bool newFlag=actualFlags[mod-1]=!actualFlags[mod-1];
-			static char const *(wh[])= {"b", "it", "outline", "underline", "shadow", "condensed", "extended"};
+			static char const *wh[]= {"b", "it", "outline", "underline", "shadow", "condensed", "extended"};
 			f << "[";
 			if (!newFlag) f << "/";
 			f << wh[mod-1] << "]";
 			if (mod<=5)
 			{
-				static uint32_t const(attrib[])= { WPS_BOLD_BIT, WPS_ITALICS_BIT, WPS_UNDERLINE_BIT, WPS_OUTLINE_BIT, WPS_SHADOW_BIT };
+				static uint32_t const attrib[]= { WPS_BOLD_BIT, WPS_ITALICS_BIT, WPS_UNDERLINE_BIT, WPS_OUTLINE_BIT, WPS_SHADOW_BIT };
 				if (newFlag)
 					font.m_attributes |= attrib[mod-1];
 				else
@@ -1378,13 +1384,13 @@ bool LotusGraph::readFMTPictName(std::shared_ptr<WPSStream> stream)
 	libwps::DebugStream f;
 
 	long pos = input->tell();
-	auto type = (int) libwps::read16(input);
+	auto type = int(libwps::read16(input));
 	if (type!=0xb7)
 	{
 		WPS_DEBUG_MSG(("LotusGraph::readFMTPictName: not a font name definition\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	if (sz!=0x68)
 	{
 		WPS_DEBUG_MSG(("LotusGraph::readFMTPictName: the zone size seems bad\n"));
@@ -1416,9 +1422,9 @@ bool LotusGraph::readFMTPictName(std::shared_ptr<WPSStream> stream)
 	for (int i=0; i<2; ++i)
 	{
 		// seems ok in wk3 files but not in wk4 files
-		auto col=(int) libwps::readU8(input);
-		auto table=(int) libwps::readU8(input);
-		auto row=(int) libwps::readU16(input);
+		auto col=int(libwps::readU8(input));
+		auto table=int(libwps::readU8(input));
+		auto row=int(libwps::readU16(input));
 		f << "C" << col << "-" << row;
 		if (table) f << "[" << table << "]";
 		if (i==0)
@@ -2051,9 +2057,16 @@ void LotusGraph::sendTextBoxWK4(std::shared_ptr<WPSStream> stream, WPSEntry cons
 			break;
 		}
 		bool send=!isButton || pos==entry.begin();
-		for (int i=0; i<dSz-1; ++i)
+		std::string text;
+		for (int i=0; i<dSz; ++i)
 		{
-			auto c=char(libwps::readU8(input));
+			auto c=i+1==dSz ? '\0' : char(libwps::readU8(input));
+			if ((c==0 || c==1) && !text.empty())
+			{
+				if (send)
+					m_listener->insertUnicodeString(libwps_tools_win::Font::unicodeString(text, fontType));
+				text.clear();
+			}
 			if (c==0)
 			{
 				if (i+2<dSz)
@@ -2084,12 +2097,11 @@ void LotusGraph::sendTextBoxWK4(std::shared_ptr<WPSStream> stream, WPSEntry cons
 						WPS_DEBUG_MSG(("LotusGraph::sendTextBox: find unexpected 1 char\n"));
 						f << "###";
 					}
-					continue;
 				}
+				continue;
 			}
 			f << c;
-			if (send)
-				m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode((unsigned char)c,fontType));
+			text.push_back(c);
 		}
 		f << ",unk=" << int(libwps::readU8(input));
 		if (input->tell()<endPos)
@@ -2198,13 +2210,13 @@ bool LotusGraph::readGraphZone(std::shared_ptr<WPSStream> stream, int zId)
 	libwps::DebugStream f;
 	float const unit=version()>=5 ? 1.f/16.f : 1.f/256.f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input) != 3)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2320,7 +2332,7 @@ bool LotusGraph::readGraphZone(std::shared_ptr<WPSStream> stream, int zId)
 		}
 		val &=0xFFFC;
 		if (val) f << "f0=" << val << ",";
-		for (int i=1; i<sz/2; ++i)   // always f0=1, other 0
+		for (long i=1; i<sz/2; ++i)   // always f0=1, other 0
 		{
 			val=int(libwps::read16(input));
 			if (val) f << "f" << i << "=" << val << ",";
@@ -2597,7 +2609,7 @@ bool LotusGraph::readGraphZone(std::shared_ptr<WPSStream> stream, int zId)
 					for (int i=0; i<5; ++i)
 					{
 						val=int(libwps::read16(input));
-						int const(expected[])= {0,0,1,1,0};
+						int const expected[]= {0,0,1,1,0};
 						if (val!=expected[i]) f2 << "l" << i << "=" << val << ",";
 					}
 					f2 << "act[id]=Z" << int(libwps::read32(input)) << ",";
