@@ -776,7 +776,8 @@ void MSWriteParser::getHeaderFooters(uint32_t first, MSWriteParserInternal::Sect
 	RVNGInputStreamPtr input = getInput();
 	int numPage = 0; // each section ends with a page break (0x0c)
 
-	unsigned firstP = 0, i;
+	size_t firstP = 0;
+	size_t i;
 
 	for (i = 0; i < m_paragraphList.size(); i++)
 	{
