@@ -117,7 +117,7 @@ protected:
 	//! read a OLE data: zone 38b
 	bool readOLEData(std::shared_ptr<WPSStream> stream);
 	//! try to read a graph header
-	bool readHeader(QuattroGraphInternal::Graph &header, std::shared_ptr<WPSStream> stream, long endPos);
+	static bool readHeader(QuattroGraphInternal::Graph &header, std::shared_ptr<WPSStream> stream, long endPos);
 
 	// ////////////////////// shape //////////////////////////////
 
@@ -138,11 +138,11 @@ protected:
 	// ////////////////////// dialog zone //////////////////////////////
 
 	//! try to read a dialog header
-	bool readHeader(QuattroGraphInternal::Dialog &header, std::shared_ptr<WPSStream> stream, long endPos);
+	static bool readHeader(QuattroGraphInternal::Dialog &header, std::shared_ptr<WPSStream> stream, long endPos);
 	//! read a dialog zone: 35e
 	bool readDialog(std::shared_ptr<WPSStream> stream);
 	//! read a unknown dialog zone: 335,343,345
-	bool readDialogUnknown(std::shared_ptr<WPSStream> stream);
+	static bool readDialogUnknown(std::shared_ptr<WPSStream> stream);
 
 	// ////////////////////// style //////////////////////////////
 	bool readFillData(WPSGraphicStyle &style, int fillType, std::shared_ptr<WPSStream> stream, long endPos);
