@@ -115,12 +115,12 @@ protected:
 	bool readCellStyle(std::shared_ptr<WPSStream> const &stream);
 
 	//! reads a view info: zone 197/198
-	bool readViewInfo(std::shared_ptr<WPSStream> const &stream);
+	static bool readViewInfo(std::shared_ptr<WPSStream> const &stream);
 
 	/* reads a cell */
-	bool readCell(std::shared_ptr<WPSStream> const &stream, Vec2i actPos, WKSContentListener::FormulaInstruction &instr, int sheetId, librevenge::RVNGString const &fName) const;
+	static bool readCell(std::shared_ptr<WPSStream> const &stream, Vec2i actPos, WKSContentListener::FormulaInstruction &instr, int sheetId, librevenge::RVNGString const &fName);
 	//! try to update the cell's format using the user format
-	void updateCellWithUserFormat(QuattroSpreadsheetInternal::Cell &cell, librevenge::RVNGString const &format);
+	static void updateCellWithUserFormat(QuattroSpreadsheetInternal::Cell &cell, librevenge::RVNGString const &format);
 	//! try to read a cell reference
 	bool readCellReference(std::shared_ptr<WPSStream> const &stream, long endPos,
 	                       QuattroFormulaInternal::CellReference &ref,
