@@ -30,6 +30,10 @@
 #  define DEBUG_PP 1
 // set to 1 to print the plc position
 #  define DEBUG_PLC_POS 1
+#else
+#  define DEBUG_FP 0
+#  define DEBUG_PP 0
+#  define DEBUG_PLC_POS 0
 #endif
 
 #include <iomanip>
@@ -87,7 +91,9 @@ enum ZoneType { Z_String=-1, Z_Header=0, Z_Footer=1, Z_Main=2, Z_Note, Z_Bookmar
 struct FontName
 {
 	//! constructor with file's version to define the default encoding */
-	explicit FontName(libwps_tools_win::Font::Type type) : m_name(""), m_type(type)
+	explicit FontName(libwps_tools_win::Font::Type type)
+		: m_name("")
+		, m_type(type)
 	{
 	}
 	//! operator<<
@@ -139,13 +145,21 @@ librevenge::RVNGString FontName::getDosName(int id)
 	return "Courier";
 }
 /** Internal: class to store font properties */
-struct Font : public WPSFont
+struct Font final : public WPSFont
 {
 	//! constructor with file's version to define the default encoding */
-	explicit Font(libwps_tools_win::Font::Type type) : WPSFont(), m_type(type),
-		m_backColor(WPSColor::white()), m_special(false), m_dlinkId(-1)
+	explicit Font(libwps_tools_win::Font::Type type)
+		: WPSFont()
+		, m_type(type)
+		, m_backColor(WPSColor::white())
+		, m_special(false)
+		, m_dlinkId(-1)
 	{
 	}
+	Font(Font const &)=default;
+	Font &operator=(Font const &)=default;
+	//! destructor
+	~Font() final;
 	//! returns a default font (Courier12) with file's version to define the default encoding */
 	static Font getDefault(libwps_tools_win::Font::Type type, int version)
 	{
@@ -171,6 +185,10 @@ struct Font : public WPSFont
 	int m_dlinkId;
 };
 
+Font::~Font()
+{
+}
+
 //! operator<< for font properties
 std::ostream &operator<<(std::ostream &o, Font const &ft)
 {
@@ -188,18 +206,19 @@ std::ostream &operator<<(std::ostream &o, Font const &ft)
 		o << "bgCol=" << ft.m_backColor << ",";
 	return o;
 }
-/** Internal: class to store paragraph properties */
-struct Paragraph : public WPSParagraph
-{
-	//! constructor
-	Paragraph() : WPSParagraph()  { }
-};
 
 /** Internal: class to store an note type */
-struct Note : public WPSEntry
+struct Note final : public WPSEntry
 {
 	//! constructor
-	Note() : WPSEntry(), m_label(""), m_error("") {}
+	Note()
+		: WPSEntry()
+		, m_label("")
+		, m_error("") {}
+	Note(Note const &)=default;
+	Note &operator=(Note const &)=default;
+	//! destructor
+	~Note() final;
 	bool isNumeric() const
 	{
 		return m_label.len()==0;
@@ -220,12 +239,21 @@ struct Note : public WPSEntry
 	std::string m_error;
 };
 
-
+Note::~Note()
+{
+}
 /** Internal: class to store an object definition */
 struct Object
 {
 	//! constructor
-	Object() : m_id(-1), m_page(0), m_origin(), m_size(), m_pos(), m_unknown(0), m_extra("") {}
+	Object()
+		: m_id(-1)
+		, m_page(0)
+		, m_origin()
+		, m_size()
+		, m_pos()
+		, m_unknown(0)
+		, m_extra("") {}
 	//! operator <<
 	friend std::ostream &operator<<(std::ostream &o, Object const &obj);
 
@@ -261,7 +289,13 @@ std::ostream &operator<<(std::ostream &o, Object const &obj)
 struct DosLink
 {
 	//! constructor
-	DosLink() : m_type(-1), m_width(-1), m_size(), m_name(""), m_pos(), m_extra("") {}
+	DosLink()
+		: m_type(-1)
+		, m_width(-1)
+		, m_size()
+		, m_name("")
+		, m_pos()
+		, m_extra("") {}
 	//! operator <<
 	friend std::ostream &operator<<(std::ostream &o, DosLink const &dlink);
 
@@ -310,7 +344,9 @@ std::ostream &operator<<(std::ostream &o, DosLink const &dlink)
 struct DateTime
 {
 	//! constructor
-	DateTime() : m_type(-1), m_extra("") {}
+	DateTime()
+		: m_type(-1)
+		, m_extra("") {}
 	//! returns a format to used with strftime
 	std::string format() const;
 	//! operator <<
@@ -400,7 +436,11 @@ enum PLCType { BTE=0, OBJECT, FTNp, FTNd, BKMK, DTTM, Unknown};
 struct DataPLC
 {
 	//! constructor
-	DataPLC(): m_name(""), m_type(Unknown), m_value(-1), m_extra() {}
+	DataPLC()
+		: m_name("")
+		, m_type(Unknown)
+		, m_value(-1)
+		, m_extra() {}
 	//! operator<<
 	friend std::ostream &operator<<(std::ostream &o, DataPLC const &plc);
 	//! the entry field name
@@ -425,10 +465,24 @@ std::ostream &operator<<(std::ostream &o, DataPLC const &plc)
 struct State
 {
 	//! constructor
-	State() : m_fontNames(), m_fontList(),  m_paragraphList(),
-		m_FDPCs(), m_FDPPs(), m_footnoteList(), m_footnoteMap(), m_bookmarkMap(), m_dosLinkList(),
-		m_main(), m_header(), m_footer(), m_otherZones(),
-		m_objectMap(), m_dateTimeMap(), m_plcList(), m_knownPLC()
+	State()
+		: m_fontNames()
+		, m_fontList()
+		, m_paragraphList()
+		, m_FDPCs()
+		, m_FDPPs()
+		, m_footnoteList()
+		, m_footnoteMap()
+		, m_bookmarkMap()
+		, m_dosLinkList()
+		, m_main()
+		, m_header()
+		, m_footer()
+		, m_otherZones()
+		, m_objectMap()
+		, m_dateTimeMap()
+		, m_plcList()
+		, m_knownPLC()
 	{}
 
 	//! the list of fonts names
@@ -436,7 +490,7 @@ struct State
 	//! the list of all font properties
 	std::vector<Font> m_fontList;
 	//! the list of all paragraph properties
-	std::vector<Paragraph> m_paragraphList;
+	std::vector<WPSParagraph> m_paragraphList;
 
 	//! the list of FDPC entries (ie list to find the font properties lists )
 	std::vector<WPSEntry> m_FDPCs;
@@ -475,8 +529,10 @@ struct State
 //////////////////////////////////////////////////////////////////////////////
 
 // constructor/destructor
-WPS4Text::WPS4Text(WPS4Parser &parser, RVNGInputStreamPtr &input) :
-	WPSTextParser(parser, input), m_listener(), m_state()
+WPS4Text::WPS4Text(WPS4Parser &parser, RVNGInputStreamPtr &input)
+	: WPSTextParser(parser, input)
+	, m_listener()
+	, m_state()
 {
 	m_state.reset(new WPS4TextInternal::State);
 }
@@ -494,12 +550,11 @@ int WPS4Text::numPages() const
 	{
 		if (libwps::readU8(m_input.get()) == 0x0C) numPage++;
 	}
-	for (std::map<long, WPS4TextInternal::Object>::const_iterator it=m_state->m_objectMap.begin();
-	        it!=m_state->m_objectMap.end(); ++it)
+	for (auto it : m_state->m_objectMap)
 	{
-		if (it->second.m_page<=numPage) continue;
-		if (it->second.m_page<=numPage+10)
-			numPage=it->second.m_page;
+		if (it.second.m_page<=numPage) continue;
+		if (it.second.m_page<=numPage+10)
+			numPage=it.second.m_page;
 		else
 		{
 			WPS_DEBUG_MSG(("WPS4Text::numPages: the number of pages seems bad\n"));
@@ -511,10 +566,9 @@ int WPS4Text::numPages() const
 // page object
 void WPS4Text::sendObjects(int page)
 {
-	for (std::map<long, WPS4TextInternal::Object>::const_iterator it=m_state->m_objectMap.begin();
-	        it!=m_state->m_objectMap.end(); ++it)
+	for (auto it : m_state->m_objectMap)
 	{
-		WPS4TextInternal::Object const &obj=it->second;
+		auto const &obj=it.second;
 		if (obj.m_page<=0 || obj.m_id<0) continue;
 		if (page>=0 && obj.m_page!=page+1) continue;
 		WPSPosition position(obj.m_origin, obj.m_size);
@@ -529,9 +583,8 @@ void WPS4Text::sendObjects(int page)
 WPSEntry WPS4Text::getHeaderEntry() const
 {
 	if (m_state->m_header.valid()) return m_state->m_header;
-	WPS4Parser::NameMultiMap const &nameMultiMap = getNameEntryMap();
-	WPS4Parser::NameMultiMap::const_iterator pos;
-	pos = nameMultiMap.find("SHdr");
+	auto const &nameMultiMap = getNameEntryMap();
+	auto pos = nameMultiMap.find("SHdr");
 	if (pos == nameMultiMap.end()) return WPSEntry();
 	WPSEntry res = pos->second;
 	res.setType("TEXT");
@@ -542,9 +595,8 @@ WPSEntry WPS4Text::getHeaderEntry() const
 WPSEntry WPS4Text::getFooterEntry() const
 {
 	if (m_state->m_footer.valid()) return m_state->m_footer;
-	WPS4Parser::NameMultiMap const &nameMultiMap = getNameEntryMap();
-	WPS4Parser::NameMultiMap::const_iterator pos;
-	pos = nameMultiMap.find("SFtr");
+	auto const &nameMultiMap = getNameEntryMap();
+	auto pos = nameMultiMap.find("SFtr");
 	if (pos == nameMultiMap.end()) return WPSEntry();
 	WPSEntry res = pos->second;
 	res.setType("TEXT");
@@ -576,7 +628,7 @@ void WPS4Text::flushExtra()
 	if (numExtra == 0) return;
 
 	m_listener->setFont(getDefaultFont());
-	m_listener->setParagraph(WPS4TextInternal::Paragraph());
+	m_listener->setParagraph(WPSParagraph());
 	m_listener->insertEOL();
 #ifdef DEBUG
 	librevenge::RVNGString message = "--------- extra text zone -------- ";
@@ -593,7 +645,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 	bool simpleString = zone.id() == WPS4TextInternal::Z_String || bookmark || dlink;
 	bool mainZone = zone.id() == WPS4TextInternal::Z_Main;
 
-	if (m_listener.get() == 0L)
+	if (m_listener.get() == nullptr)
 	{
 		WPS_DEBUG_MSG(("WPS4Text::readText can not find the listener\n"));
 		return false;
@@ -615,14 +667,14 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			}
 			else
 			{
-				int w = int(72.0*mainParser().pageWidth())/numCols;
+				int w = int(72*mainParser().pageWidth())/numCols;
 				std::vector<int> width;
 				width.resize(size_t(numCols), w);
 				m_listener->openSection(width,librevenge::RVNG_POINT);
 			}
 		}
 	}
-	std::vector<DataFOD>::iterator FODs_iter = m_FODList.begin();
+	auto FODs_iter = m_FODList.begin();
 
 	// update the property to correspond to the text
 	int prevFId = -1, prevPId = -1;
@@ -635,7 +687,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 
 	for (; FODs_iter!= m_FODList.end(); ++FODs_iter)
 	{
-		DataFOD const &fod = *(FODs_iter);
+		auto const &fod = *(FODs_iter);
 		if (fod.m_pos >= zone.begin()) break;
 
 		int id = (*FODs_iter).m_id;
@@ -652,7 +704,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 	if (prevPId != -1)
 		m_listener->setParagraph(m_state->m_paragraphList[size_t(prevPId)]);
 	else
-		m_listener->setParagraph(WPS4TextInternal::Paragraph());
+		m_listener->setParagraph(WPSParagraph());
 
 	if (dlink)
 	{
@@ -677,7 +729,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 		}
 		else
 		{
-			DataFOD fod = *(FODs_iter);
+			auto fod = *(FODs_iter);
 			actPos = first ? zone.begin() : fod.m_pos;
 			if (long(actPos) >= zone.end()) break;
 			first = false;
@@ -709,7 +761,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 				if (fId >= 0)
 					m_listener->setParagraph(m_state->m_paragraphList[size_t(fId)]);
 				else
-					m_listener->setParagraph(WPS4TextInternal::Paragraph());
+					m_listener->setParagraph(WPSParagraph());
 #if DEBUG_PP
 				f << "[";
 				if (fId >= 0) f << "P" << fId << ":" << m_state->m_paragraphList[size_t(fId)] << "]";
@@ -748,13 +800,40 @@ bool WPS4Text::readText(WPSEntry const &zone)
 		m_input->seek(actPos, librevenge::RVNG_SEEK_SET);
 		std::string chaine("");
 		long len = lastPos-actPos;
-		for (long i = len; i>0; i--)
+		std::string text;
+		for (long i = len; i>=0; i--)
 		{
 			long pos = m_input->tell();
-			uint8_t readVal = libwps::readU8(m_input);
+			uint8_t readVal = i==0 ? uint8_t('\0') : libwps::readU8(m_input);
+			bool isSimpleField=false;
+			if (readVal=='&' && simpleString && pos+2 <= lastPos)
+			{
+				int nextVal = libwps::readU8(m_input);
+				m_input->seek(-1, librevenge::RVNG_SEEK_CUR);
+				switch (nextVal)   // check me
+				{
+				case 'p':
+				case 'P':
+				case 'd':
+				case 'D':
+				case 't':
+				case 'T':
+				case 'f':
+				case 'F':
+					isSimpleField=true;
+					break;
+				default:
+					break;
+				}
+			}
+			if ((readVal<0x10 || readVal==0x11 || readVal==0x12 || readVal==0x1f || (version()<=2 && readVal==0xca) || isSimpleField) && !text.empty())
+			{
+				m_listener->insertUnicodeString(libwps_tools_win::Font::unicodeString(text, actFont.m_type));
+				text.clear();
+			}
 			if (0x00 == readVal)
 			{
-				if (i != 1)
+				if (i > 1)
 				{
 					WPS_DEBUG_MSG(("WPS4Text::readText: find some unexpected 0 character\n"));
 					// probably an error, but we can ignore id
@@ -762,7 +841,6 @@ bool WPS4Text::readText(WPSEntry const &zone)
 				}
 				continue;
 			}
-
 			chaine += char(readVal);
 			switch (readVal)
 			{
@@ -803,7 +881,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			{
 				if (m_state->m_objectMap.find(actPos) == m_state->m_objectMap.end())
 				{
-					WPS_DEBUG_MSG(("WPS4Text::readText: can not find object for position : %lX\n", (unsigned long) actPos));
+					WPS_DEBUG_MSG(("WPS4Text::readText: can not find object for position : %lX\n", static_cast<unsigned long>(actPos)));
 				}
 				else
 				{
@@ -820,11 +898,11 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			{
 				if (m_state->m_dateTimeMap.find(actPos) == m_state->m_dateTimeMap.end())
 				{
-					WPS_DEBUG_MSG(("WPS4Text::readText: can not find date/time for position : %lX\n", (unsigned long) actPos));
+					WPS_DEBUG_MSG(("WPS4Text::readText: can not find date/time for position : %lX\n", static_cast<unsigned long>(actPos)));
 				}
 				else
 				{
-					WPS4TextInternal::DateTime const &form = m_state->m_dateTimeMap[actPos];
+					auto const &form = m_state->m_dateTimeMap[actPos];
 					std::string format = form.format();
 					if (format.length())
 					{
@@ -834,7 +912,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 					}
 					else
 					{
-						WPS_DEBUG_MSG(("WPS4Text::readText: unknown date/time format for position : %lX\n", (unsigned long) actPos));
+						WPS_DEBUG_MSG(("WPS4Text::readText: unknown date/time format for position : %lX\n", static_cast<unsigned long>(actPos)));
 					}
 				}
 				break;
@@ -844,12 +922,12 @@ bool WPS4Text::readText(WPSEntry const &zone)
 				// ok if this is the first character of the footnote definition
 				if (zone.id() == WPS4TextInternal::Z_Note) break;
 				if (m_state->m_footnoteMap.find(pos) == m_state->m_footnoteMap.end() ||
-				        m_state->m_footnoteMap[pos] == 0L)
+				        m_state->m_footnoteMap[pos] == nullptr)
 				{
 					WPS_DEBUG_MSG(("WPS4Text::readText:do not find the footnote zone\n"));
 					break;
 				}
-				WPS4TextInternal::Note const &note = *m_state->m_footnoteMap[pos];
+				auto const &note = *m_state->m_footnoteMap[pos];
 				mainParser().createNote(note, note.m_label);
 				break;
 			}
@@ -876,7 +954,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 			case 0x1F: // optional hyphen
 				break;
 			case '&':
-				if (simpleString && pos+2 <= lastPos)
+				if (isSimpleField)
 				{
 					int nextVal = libwps::readU8(m_input);
 					bool done = true;
@@ -921,7 +999,8 @@ bool WPS4Text::readText(WPSEntry const &zone)
 						break;
 					}
 				}
-				m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(readVal, actFont.m_type));
+				if (readVal)
+					text.push_back(char(readVal));
 				break;
 			}
 		}
@@ -944,8 +1023,7 @@ bool WPS4Text::readText(WPSEntry const &zone)
 ////////////////////////////////////////////////////////////
 bool WPS4Text::readEntries()
 {
-	WPS4Parser::NameMultiMap &nameMultiMap = getNameEntryMap();
-	WPS4Parser::NameMultiMap::iterator pos;
+	auto &nameMultiMap = getNameEntryMap();
 
 	libwps::DebugStream f;
 	long actPos = m_input->tell();
@@ -954,7 +1032,7 @@ bool WPS4Text::readEntries()
 	int textLimits[4];
 	// look like begin of text : end of header/end of footer/end text
 	// but sometimes the zones overlaps !!!
-	for (int i = 0; i < 4; ++i) textLimits[i] = libwps::read32(m_input);
+	for (int &textLimit : textLimits) textLimit = libwps::read32(m_input);
 
 	bool first = true, ok = true;
 	long lastPos = textLimits[0] < 0x100 ? 0x100 : textLimits[0];
@@ -1026,7 +1104,7 @@ bool WPS4Text::readEntries()
 	}
 
 	/* stream offset to end of file */
-	long eof = (long) libwps::readU32(m_input);
+	auto eof = long(libwps::readU32(m_input));
 
 	if (m_textPositions.end() > eof)
 	{
@@ -1050,11 +1128,11 @@ bool WPS4Text::readEntries()
 
 	m_input->seek(newPos, librevenge::RVNG_SEEK_SET);
 
-	static char const * (zName[]) =
+	static char const   *zName[] =
 	{ "BTEC", "BTEP", "SHdr", "SFtr", "DLINK", "FTNp", "FTNd", "BKMK", "FONT" };
 
-	for (int i = 0; i < 9; ++i)
-		mainParser().parseEntry(zName[i]);
+	for (auto &i : zName)
+		mainParser().parseEntry(i);
 
 	return true;
 }
@@ -1064,11 +1142,10 @@ bool WPS4Text::readEntries()
 ////////////////////////////////////////////////////////////
 bool WPS4Text::readStructures()
 {
-	WPS4Parser::NameMultiMap &nameMultiMap = getNameEntryMap();
-	WPS4Parser::NameMultiMap::iterator pos;
+	auto &nameMultiMap = getNameEntryMap();
 
 	// first find the font name
-	pos = nameMultiMap.find("FONT");
+	auto pos = nameMultiMap.find("FONT");
 	if (pos != nameMultiMap.end()) readFontNames(pos->second);
 
 	// now find the character and paragraph properties
@@ -1080,18 +1157,16 @@ bool WPS4Text::readStructures()
 	}
 
 	/* read character FODs (FOrmatting Descriptors) */
-	size_t numFDP = m_state->m_FDPCs.size();
 	std::vector<DataFOD> fdps;
-	for (size_t i = 0; i < numFDP; ++i)
-		readFDP(m_state->m_FDPCs[i], fdps, (FDPParser)&WPS4Text::readFont);
+	for (auto const &fdp : m_state->m_FDPCs)
+		readFDP(fdp, fdps, static_cast<FDPParser>(&WPS4Text::readFont));
 	m_FODList = mergeSortedFODLists(fdps, m_FODList);
 
 
 	/* read paragraphs FODs (FOrmatting Descriptors) */
 	fdps.resize(0);
-	numFDP = m_state->m_FDPPs.size();
-	for (size_t i = 0; i < numFDP; ++i)
-		readFDP(m_state->m_FDPPs[i], fdps, (FDPParser)&WPS4Text::readParagraph);
+	for (auto const &fdp : m_state->m_FDPPs)
+		readFDP(fdp, fdps, static_cast<FDPParser>(&WPS4Text::readParagraph));
 	m_FODList = mergeSortedFODLists(fdps, m_FODList);
 
 	/* read the object structures */
@@ -1187,8 +1262,8 @@ bool WPS4Text::findFDPStructures(int which)
 	char const *indexName = which ? "BTEC" : "BTEP";
 	char const *sIndexName = which ? "FDPC" : "FDPP";
 
-	WPS4Parser::NameMultiMap &nameMultiMap =getNameEntryMap();
-	WPS4Parser::NameMultiMap::iterator pos = nameMultiMap.find(indexName);
+	auto &nameMultiMap =getNameEntryMap();
+	auto pos = nameMultiMap.find(indexName);
 	if (pos == nameMultiMap.end()) return false;
 
 	std::vector<long> textPtrs;
@@ -1220,7 +1295,7 @@ bool WPS4Text::findFDPStructuresByHand(int which)
 	char const *indexName = which ? "FDPC" : "FDPP";
 	WPS_DEBUG_MSG(("WPS4Text::findFDPStructuresByHand: need to create %s list by hand \n", indexName));
 
-	std::vector<WPSEntry> &zones = which ? m_state->m_FDPCs : m_state->m_FDPPs;
+	auto &zones = which ? m_state->m_FDPCs : m_state->m_FDPPs;
 	zones.resize(0);
 
 	long debPos;
@@ -1228,14 +1303,14 @@ bool WPS4Text::findFDPStructuresByHand(int which)
 	{
 		// hack: each fdp block is aligned with 0x80,
 		//       and appears consecutively just after the text
-		uint32_t pnChar = uint32_t((m_textPositions.end()+127)>>7);
+		auto pnChar = uint32_t((m_textPositions.end()+127)>>7);
 		/* sanity check */
 		if (0 == pnChar)
 		{
 			WPS_DEBUG_MSG(("WPS4Text::findFDPStructuresByHand: pnChar is 0, so file may be corrupt\n"));
 			throw libwps::ParseException();
 		}
-		debPos = 0x80 * (long) pnChar;
+		debPos = 0x80 * long(pnChar);
 	}
 	else
 	{
@@ -1275,7 +1350,7 @@ bool WPS4Text::findFDPStructuresByHand(int which)
 		if (nbElt != 1)
 			m_input->seek(4*nbElt-4, librevenge::RVNG_SEEK_CUR);
 
-		long newPos = (long) libwps::readU32(m_input);
+		auto newPos = long(libwps::readU32(m_input));
 		if (newPos < lastPos || newPos > m_textPositions.end())
 		{
 			WPS_DEBUG_MSG(("WPS4Text: find incorrect linking while parsing the %s\n", indexName));
@@ -1316,7 +1391,7 @@ bool WPS4Text::defDataParser(long, long, int, long endPos, std::string &mess)
 			val = libwps::readU16(m_input);
 			break;
 		case 4:
-			val = (long) libwps::readU32(m_input);
+			val = long(libwps::readU32(m_input));
 			break;
 		default:
 			break;
@@ -1338,7 +1413,7 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 
 	long endPos = entry.end();
 	int nFonts = 0;
-	libwps_tools_win::Font::Type docType=mainParser().getDefaultFontType();
+	auto docType=mainParser().getDefaultFontType();
 	while (m_input->tell() < endPos)
 	{
 		long actPos;
@@ -1351,14 +1426,14 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 		if (m_state->m_fontNames.find(font_number) != m_state->m_fontNames.end())
 		{
 			WPS_DEBUG_MSG(("WPS4Text::readFontNames: at position 0x%lx: font number %i duplicated\n",
-			               (unsigned long)(m_input->tell()-2), font_number));
+			               static_cast<unsigned long>(m_input->tell()-2), font_number));
 			throw libwps::ParseException();
 		}
 
-		f << "Font" << nFonts++ << ": id=" << (int)font_number << ", ";
+		f << "Font" << nFonts++ << ": id=" << int(font_number) << ", ";
 		//fixme: what is this byte? maybe a font class
 		uint8_t unknown_byte = libwps::readU8(m_input);
-		f << "unk=" << (int)unknown_byte << ", ";
+		f << "unk=" << int(unknown_byte) << ", ";
 
 		librevenge::RVNGString s;
 		uint8_t nChar = libwps::readU8(m_input);
@@ -1379,12 +1454,12 @@ bool WPS4Text::readFontNames(WPSEntry const &entry)
 				if (first)
 				{
 					first = false;
-					WPS_DEBUG_MSG(("WPS4Text:readFontNames find odd caracters in font name : %d\n", (int) val));
+					WPS_DEBUG_MSG(("WPS4Text:readFontNames find odd caracters in font name : %d\n", int(val)));
 				}
-				f << "##oddC=" << (unsigned int) val << ", ";
+				f << "##oddC=" << int(val) << ", ";
 			}
 		}
-		libwps_tools_win::Font::Type fType=libwps_tools_win::Font::getFontType(s);
+		auto fType=libwps_tools_win::Font::getFontType(s);
 		if (fType==libwps_tools_win::Font::UNKNOWN)
 			fType=docType;
 		WPS4TextInternal::FontName font(fType);
@@ -1445,7 +1520,7 @@ bool WPS4Text::readFont(long endPos, int &id, std::string &mess)
 
 		if (m_state->m_fontNames.find(font_n) != m_state->m_fontNames.end())
 		{
-			WPS4TextInternal::FontName const &fontName=m_state->m_fontNames.find(font_n)->second;
+			auto const &fontName=m_state->m_fontNames.find(font_n)->second;
 			font.m_name=fontName.m_name;
 			font.m_type=fontName.m_type;
 		}
@@ -1587,7 +1662,7 @@ bool WPS4Text::readDosLink(WPSEntry const &entry)
 		case 0x81: // picture ?
 		{
 			long dim[2];
-			for (int i = 0; i < 2; ++i) dim[i] = libwps::readU16(m_input);
+			for (long &i : dim) i = libwps::readU16(m_input);
 			link.m_size = Vec2f(float(dim[0])/1440.f, float(dim[1])/1440.f);
 			val = libwps::readU16(m_input); // always 0
 			if (val) f << "g0=" << val << ",";
@@ -1602,7 +1677,7 @@ bool WPS4Text::readDosLink(WPSEntry const &entry)
 			link.m_pos.setBegin(m_input->tell());
 			while (!m_input->isEnd() && long(m_input->tell()) < endPos)
 			{
-				char c = char(libwps::readU8(m_input));
+				auto c = char(libwps::readU8(m_input));
 				if (!c)
 				{
 					m_input->seek(-1, librevenge::RVNG_SEEK_CUR);
@@ -1639,7 +1714,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 	long actPos = m_input->tell();
 	long size = endPos - actPos;
 
-	WPS4TextInternal::Paragraph pp;
+	WPSParagraph pp;
 	if (size && size < 3)
 	{
 		WPS_DEBUG_MSG(("WPS4Text:readParagraph:(sz=%ld)\n", size));
@@ -1921,7 +1996,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 			int flag = libwps::readU8(m_input);
 			if (flag) f << "#tabsFl=" << flag << ",";
 			size_t nItem = libwps::readU8(m_input);
-			if ((unsigned long)nVal != 2 + 3*nItem)
+			if (size_t(nVal) != 2 + 3*nItem)
 			{
 				ok = false;
 				break;
@@ -2037,7 +2112,7 @@ bool WPS4Text::readParagraph(long endPos, int &id, std::string &mess)
 			m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 			f << "###v" << v<<"=" <<std::hex;
 			while (m_input->tell() < endPos)
-				f << (int) libwps::readU8(m_input) << ",";
+				f << int(libwps::readU8(m_input)) << ",";
 			break;
 		}
 
@@ -2122,15 +2197,14 @@ bool WPS4Text::readFootNotes(WPSEntry const &ftnD, WPSEntry const &ftnP)
 			{
 				WPS_DEBUG_MSG
 				(("WPS4Text: error: can not create footnotes zone, found %lx and %lx\n",
-				  (unsigned long) m_state->m_footnoteList[j].end(),(unsigned long) m_state->m_footnoteList[j-1].begin()));
+				  static_cast<unsigned long>(m_state->m_footnoteList[j].end()),
+				  static_cast<unsigned long>(m_state->m_footnoteList[j-1].begin())));
 
 				m_state->m_footnoteList.resize(0);
 				return false;
 			}
 
-			WPS4TextInternal::Note tmpZ = m_state->m_footnoteList[j];
-			m_state->m_footnoteList[j] = m_state->m_footnoteList[j-1];
-			m_state->m_footnoteList[j-1] = tmpZ;
+			std::swap(m_state->m_footnoteList[j], m_state->m_footnoteList[j-1]);
 
 			int pos = corresp[j];
 			corresp[j] = corresp[j-1];
@@ -2140,7 +2214,7 @@ bool WPS4Text::readFootNotes(WPSEntry const &ftnD, WPSEntry const &ftnP)
 	// ok, we can create the map, ...
 	for (size_t i = 0; i < size_t(numFootNotes); ++i)
 	{
-		size_t id = size_t(corresp[i]);
+		auto id = size_t(corresp[i]);
 		WPS4TextInternal::Note &z = m_state->m_footnoteList[id];
 		if (id < noteTypes.size())
 		{
@@ -2166,7 +2240,7 @@ bool WPS4Text::footNotesDataParser(long /*bot*/, long /*eot*/, int id,
 	}
 	libwps::DebugStream f;
 	WPS4TextInternal::Note note;
-	int type = libwps::readU16(m_input);
+	auto type = libwps::readU16(m_input);
 	if (type & 1)
 	{
 		if (type != 1)
@@ -2175,19 +2249,7 @@ bool WPS4Text::footNotesDataParser(long /*bot*/, long /*eot*/, int id,
 	else if (type == 0 || type > 20)
 		f << "###char,";
 	else
-	{
-		int numC = type/2;
-		librevenge::RVNGString label("");
-		libwps_tools_win::Font::Type actType = mainParser().getDefaultFontType();
-		for (int i=0; i < numC; ++i)
-		{
-			unsigned char c = libwps::readU8(m_input);
-			libwps::appendUnicode(uint32_t(libwps_tools_win::Font::unicode(c, actType)),label);
-			if (c < 0x20)
-				f << "#(" << std::hex << int(c) << std::dec << ")";
-		}
-		note.m_label = label;
-	}
+		note.m_label=libwps_tools_win::Font::unicodeString(m_input.get(), type/2, mainParser().getDefaultFontType());
 	note.m_error=f.str();
 	if (id >= int(m_state->m_footnoteList.size()))
 		m_state->m_footnoteList.resize(size_t(id+1));
@@ -2222,7 +2284,7 @@ bool WPS4Text::bkmkDataParser(long bot, long /*eot*/, int /*id*/,
 
 	for (int i = 0; i < 16; ++i)
 	{
-		char c = char(libwps::readU8(m_input));
+		auto c = char(libwps::readU8(m_input));
 		if (c == '\0') break;
 		mess += c;
 	}
@@ -2265,8 +2327,8 @@ bool WPS4Text::objectDataParser(long bot, long /*eot*/, int id,
 		if (v) f << "unkn1:" << i << "=" << v << ",";
 	}
 	float dim[4];
-	for (int i = 0; i < 4; ++i)
-		dim[i] =float(libwps::read16(m_input)/1440.);
+	for (float &i : dim)
+		i =float(libwps::read16(m_input)/1440.);
 
 	// CHECKME: the next two sizes are often simillar,
 	//         maybe the first one is the original size and the second
@@ -2276,8 +2338,8 @@ bool WPS4Text::objectDataParser(long bot, long /*eot*/, int id,
 	WPS4TextInternal::Object obj;
 	obj.m_size = Vec2f(dim[2], dim[3]); // CHECKME: unit
 
-	long size = (long) libwps::readU32(m_input);
-	long pos = (long) libwps::readU32(m_input);
+	auto size = long(libwps::readU32(m_input));
+	auto pos = long(libwps::readU32(m_input));
 
 	actPos = m_input->tell();
 	if (pos >= 0 && size > 0 && mainParser().checkFilePosition(pos+size))
@@ -2299,7 +2361,7 @@ bool WPS4Text::objectDataParser(long bot, long /*eot*/, int id,
 	}
 
 	m_input->seek(actPos, librevenge::RVNG_SEEK_SET);
-	int val =int(libwps::read16(m_input)); // small number, probably an id
+	auto val =int(libwps::read16(m_input)); // small number, probably an id
 	if (val) f << "id=" << val << ",";
 	for (int i = 0; i < 2; ++i)
 	{
@@ -2308,7 +2370,7 @@ bool WPS4Text::objectDataParser(long bot, long /*eot*/, int id,
 	}
 	obj.m_page =int(libwps::read16(m_input));
 	int origin[2];
-	for (int i=0; i<2; ++i) origin[i] =int(libwps::read16(m_input));
+	for (int &i : origin) i =int(libwps::read16(m_input));
 	obj.m_origin=Vec2f(float(origin[0]/1440.),float(origin[1]/1440.));
 	val =int(libwps::read16(m_input)); // can be big
 	if (val) f << "f2=" << val << ",";
@@ -2394,7 +2456,10 @@ struct PLC
 	//! constructor
 	PLC(PLCType w= WPS4TextInternal::Unknown, Position p=P_UNKNOWN, Type t=T_UNKNOWN, unsigned char tChar='\0', int f=1) :
 		m_type(w), m_pos(p), m_contentType(t), m_textChar(tChar), m_cstFactor(f) {}
-
+	PLC(PLC const &orig)=default;
+	PLC(PLC &&orig)=default;
+	PLC &operator=(PLC const &orig)=default;
+	PLC &operator=(PLC &&orig)=default;
 	//! PLC type
 	PLCType m_type;
 	//! the way to define the text positions
@@ -2409,7 +2474,8 @@ struct PLC
 	int m_cstFactor;
 };
 
-KnownPLC::KnownPLC() : m_knowns()
+KnownPLC::KnownPLC()
+	: m_knowns()
 {
 	createMapping();
 }
@@ -2420,7 +2486,7 @@ KnownPLC::~KnownPLC()
 
 PLC KnownPLC::get(std::string const &name)
 {
-	std::map<std::string, PLC>::iterator pos = m_knowns.find(name);
+	auto pos = m_knowns.find(name);
 	if (pos == m_knowns.end()) return PLC();
 	return pos->second;
 }
@@ -2452,7 +2518,7 @@ bool WPS4Text::readPLC
 	listValues.resize(0);
 	long size = zone.length();
 	if (zone.begin() <= 0 || size < 8) return false;
-	WPS4PLCInternal::PLC plcType = m_state->m_knownPLC.get(zone.type());
+	auto plcType = m_state->m_knownPLC.get(zone.type());
 
 	libwps::DebugStream f;
 	ascii().addPos(zone.begin());
@@ -2464,7 +2530,7 @@ bool WPS4Text::readPLC
 	f << "pos=(";
 	while (numElt*4+4 <= unsigned(size))
 	{
-		long newPos = (long) libwps::readU32(m_input);
+		auto newPos = long(libwps::readU32(m_input));
 		if (plcType.m_pos == WPS4PLCInternal::PLC::P_UNKNOWN)
 		{
 			if (newPos < m_textPositions.begin())
@@ -2543,7 +2609,7 @@ bool WPS4Text::readPLC
 	// ok we have some data
 	bool ok = true;
 	long pos = m_input->tell();
-	WPS4Text::DataParser pars = parser;
+	auto pars = parser;
 	if ((dataSize == 3 || dataSize > 4) && !pars)
 		pars = &WPS4Text::defDataParser;
 
@@ -2562,7 +2628,7 @@ bool WPS4Text::readPLC
 				plc.m_value = libwps::readU16(m_input);
 				break;
 			case 4:
-				plc.m_value = (long) libwps::readU32(m_input);
+				plc.m_value = long(libwps::readU32(m_input));
 				break;
 			default:
 				WPS_DEBUG_MSG(("WPS4Text:readPLC: unexpected PLC size\n"));
