@@ -42,12 +42,15 @@
 namespace WPS8TextStyleInternal
 {
 /** Internal: class to store  font properties */
-struct Font : public WPSFont
+struct Font final : public WPSFont
 {
 	//! constructor
 	Font()
 		: WPSFont()
 		, m_special() { }
+	Font(Font const &)=default;
+	//! destructor
+	~Font() final;
 	static Font def()
 	{
 		Font res;
@@ -75,6 +78,10 @@ protected:
 	WPS8TextStyle::FontData m_special;
 };
 
+Font::~Font()
+{
+}
+
 //! operator<< for a font
 std::ostream &operator<<(std::ostream &o, Font const &ft)
 {
@@ -189,8 +196,8 @@ bool WPS8TextStyle::readStructures()
 			findFDPStructuresByHand(st, zones);
 
 		std::vector<WPS8Text::DataFOD> fdps;
-		WPS8Text::FDPParser parser = st==0  ? (WPS8Text::FDPParser) &WPS8Text::readParagraph
-		                             : (WPS8Text::FDPParser) &WPS8Text::readFont;
+		WPS8Text::FDPParser parser = st==0  ? WPS8Text::FDPParser(&WPS8Text::readParagraph)
+		                             : WPS8Text::FDPParser(&WPS8Text::readFont);
 		for (auto const &z : zones)
 			m_mainParser.readFDP(z, fdps, parser);
 		m_mainParser.m_FODList = m_mainParser.mergeSortedFODLists(m_mainParser.m_FODList, fdps);
@@ -230,8 +237,8 @@ bool WPS8TextStyle::readFontNames(WPSEntry const &entry)
 	long debPos = entry.begin();
 	m_input->seek(debPos, librevenge::RVNG_SEEK_SET);
 
-	auto len = (long) libwps::readU32(m_input); // len + 0x14 = size
-	auto n_fonts = (size_t) libwps::readU32(m_input);
+	auto len = long(libwps::readU32(m_input)); // len + 0x14 = size
+	auto n_fonts = size_t(libwps::readU32(m_input));
 
 	if (long(4*n_fonts) > len)
 	{
@@ -266,17 +273,17 @@ bool WPS8TextStyle::readFontNames(WPSEntry const &entry)
 		debPos = m_input->tell();
 		if (debPos+6 > long(pageEnd)) break;
 
-		auto string_size = (int) libwps::readU16(m_input);
+		auto string_size = int(libwps::readU16(m_input));
 		if (debPos+2*string_size+6 > long(pageEnd)) break;
 
 		librevenge::RVNGString s;
 		for (; string_size>0; string_size--) // checkme
-			s.append((char) libwps::readU16(m_input));
+			s.append(char(libwps::readU16(m_input)));
 
 		f.str("");
 		f << "FONT("<<m_state->m_fontNames.size()<<"): " << s.cstr();
 		f << ", unkn=(";
-		for (int i = 0; i < 4; i++) f << (int) libwps::read8(m_input) << ", ";
+		for (int i = 0; i < 4; i++) f << int(libwps::read8(m_input)) << ", ";
 		f << ")";
 		ascii().addPos(debPos);
 		ascii().addNote(f.str().c_str());
@@ -310,7 +317,7 @@ bool WPS8TextStyle::readSGP(WPSEntry const &entry)
 
 	if (length < 2)
 	{
-		WPS_DEBUG_MSG(("WPS8TextStyle::readSGP: warning: SGP length=0x%lx\n", (unsigned long) length));
+		WPS_DEBUG_MSG(("WPS8TextStyle::readSGP: warning: SGP length=0x%lx\n", static_cast<unsigned long>(length)));
 		return false;
 	}
 
@@ -462,7 +469,7 @@ bool WPS8TextStyle::readFont(long endPos, int &id, std::string &mess)
 				f << "#engrave=false,";
 			break;
 		case 0x18: // 0 or 0.25
-			f << "##f24(inches)=" << float(data.m_value)/914400.f << ",";
+			f << "##f24(inches)=" << double(data.m_value)/914400 << ",";
 			break;
 		case 0x1b:   // -3175
 			if (data.m_value == -3175) f << "##f27,";
@@ -555,7 +562,7 @@ bool WPS8TextStyle::readFont(long endPos, int &id, std::string &mess)
 				f << "###f24=[" << data << "]";
 				break;
 			}
-			auto fontId = (uint8_t)data.m_recursData[0].m_value;
+			auto fontId = uint8_t(data.m_recursData[0].m_value);
 			if (fontId < m_state->m_fontNames.size())
 
 				font.m_name = m_state->m_fontNames[fontId];
@@ -704,23 +711,23 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 			}
 			break;
 		case 0x6:
-			para.m_listLevel.m_labelIndent = float(data.m_value)/914400.f;
+			para.m_listLevel.m_labelIndent = double(data.m_value)/914400;
 			break;
 		case 0xc: // first line indentation (6*152400 unit = 1 inches)
-			para.m_margins[0] = float(data.m_value)/914400.f;
+			para.m_margins[0] = double(data.m_value)/914400;
 			break;
 		case 0xd: // left indentation
-			para.m_margins[1] = float(data.m_value)/914400.f;
+			para.m_margins[1] = double(data.m_value)/914400;
 			break;
 		case 0xe: // right indentation
-			para.m_margins[2] = float(data.m_value)/914400.f;
+			para.m_margins[2] = double(data.m_value)/914400;
 			break;
 
 		case 0x12: // before line spacing 152400 -> 1 line
-			para.m_spacings[1] = float(data.m_value)/152400.f;
+			para.m_spacings[1] = double(data.m_value)/152400;
 			break;
 		case 0x13: // after line spacing 152400 -> 1 line
-			para.m_spacings[2] = float(data.m_value)/152400.f;
+			para.m_spacings[2] = double(data.m_value)/152400;
 			break;
 		// case 0x15(type22): one time with value 0x29
 		case 0x14:
@@ -784,7 +791,7 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 				para.m_listLevel.m_bullet.clear();
 				uint32_t code = 0;
 				if (data.m_value <= 255)
-					code = uint32_t(libwps_tools_win::Font::unicode((unsigned char)data.m_value, libwps_tools_win::Font::WIN3_WINGDINGS));
+					code = uint32_t(libwps_tools_win::Font::unicode(static_cast<unsigned char>(data.m_value), libwps_tools_win::Font::WIN3_WINGDINGS));
 				if (!code)
 				{
 					WPS_DEBUG_MSG(("WPS8TextStyle::readParagraph: can not find bullet code\n"));
@@ -927,7 +934,7 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 					        && actTab < numTabs)
 					{
 						para.m_tabs[size_t(actTab)].m_alignment = WPSTabStop::LEFT;
-						para.m_tabs[size_t(actTab)].m_position =  float(mDataC.m_value)/914400.f;
+						para.m_tabs[size_t(actTab)].m_position =  double(mDataC.m_value)/914400;
 
 						actTab++;
 						continue;
@@ -984,7 +991,7 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 					}
 					if (mDataC.isNumber() && wTab == actTab-1 && what == 2)
 					{
-						para.m_tabs[size_t(actTab-1)].m_leaderCharacter = (uint16_t) mDataC.m_value;
+						para.m_tabs[size_t(actTab-1)].m_leaderCharacter = uint16_t(mDataC.m_value);
 						continue;
 					}
 					f << "###tabData:fl" << actTab << "=" << mDataC << ",";
@@ -1006,13 +1013,13 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 		case 0x34: // interline line spacing 8*152400 -> normal, sinon *2
 		{
 			float lines = float(data.m_value)/1219200.f;
-			if (lines < 0.5)
+			if (lines < 0.5f)
 			{
 				// find in one file some bogus line spacing between 0.2 and 0.3
 				f << "###lineSpacing = " << lines << ",";
-				lines = 1.0;
+				lines = 1;
 			}
-			para.setInterline(lines, librevenge::RVNG_PERCENT);
+			para.setInterline(double(lines), librevenge::RVNG_PERCENT);
 			break;
 		}
 
@@ -1033,7 +1040,7 @@ bool WPS8TextStyle::readParagraph(long endPos, int &id, std::string &mess)
 		para.m_margins[0]+=para.m_margins[1];
 		para.m_listLevel.m_labelIndent = para.m_margins[0];
 	}
-	id = (int) m_state->m_paragraphList.size();
+	id = int(m_state->m_paragraphList.size());
 	m_state->m_paragraphList.push_back(para);
 
 	f.str("");
@@ -1072,7 +1079,7 @@ bool WPS8TextStyle::readSTSH(WPSEntry const &entry)
 
 	if (length < 20)
 	{
-		WPS_DEBUG_MSG(("WPS8TextStyle::readSTSH: warning: STSH length=0x%lx\n", (unsigned long) length));
+		WPS_DEBUG_MSG(("WPS8TextStyle::readSTSH: warning: STSH length=0x%lx\n", static_cast<unsigned long>(length)));
 		return false;
 	}
 
@@ -1138,7 +1145,7 @@ bool WPS8TextStyle::readSTSH(WPSEntry const &entry)
 		f << "):";
 
 		m_input->seek(pos, librevenge::RVNG_SEEK_SET);
-		auto size = (int) libwps::readU16(m_input);
+		auto size = int(libwps::readU16(m_input));
 		bool correct = true;
 		if (2*size + 2 + type != length) correct = false;
 		else
@@ -1148,7 +1155,7 @@ bool WPS8TextStyle::readSTSH(WPSEntry const &entry)
 			case 4:
 			{
 				librevenge::RVNGString str;
-				if (size) m_mainParser.readString(m_input, 2*size, str);
+				if (size) WPS8Text::readString(m_input, 2*size, str);
 				f << "'" << str.cstr() << "'";
 				m_input->seek(pos+2+2*size, librevenge::RVNG_SEEK_SET);
 				f << ", unkn=" << libwps::read32(m_input);
@@ -1157,7 +1164,7 @@ bool WPS8TextStyle::readSTSH(WPSEntry const &entry)
 			case 0:
 			{
 				WPS8Struct::FileData mainData;
-				auto dataSz = (int) libwps::readU16(m_input);
+				auto dataSz = int(libwps::readU16(m_input));
 				if (dataSz+2 != 2*size)
 				{
 					correct = false;
