@@ -1337,9 +1337,9 @@ void WPSContentListener::_handleFrameParameters
 		                              "paragraph" : "paragraph-content");
 		propList.insert("style:vertical-rel", posRel);
 		propList.insert("style:horizontal-rel", posRel);
-		float w = float (m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft
-		                 - m_ps->m_pageMarginRight - m_ps->m_paragraph.m_margins[1]
-		                 - m_ps->m_paragraph.m_margins[2]);
+		auto w = float (m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft
+		                - m_ps->m_pageMarginRight - m_ps->m_paragraph.m_margins[1]
+		                - m_ps->m_paragraph.m_margins[2]);
 		w *= inchFactor;
 		switch (pos.m_xPos)
 		{
@@ -1389,8 +1389,8 @@ void WPSContentListener::_handleFrameParameters
 		// Page position seems to do not use the page margin...
 		propList.insert("text:anchor-type", "page");
 		if (pos.page() > 0) propList.insert("text:anchor-page-number", pos.page());
-		float w = float(m_ps->m_pageFormWidth);
-		float h = float(m_ps->m_pageFormLength);
+		auto w = float(m_ps->m_pageFormWidth);
+		auto h = float(m_ps->m_pageFormLength);
 		w *= inchFactor;
 		h *= inchFactor;
 
