@@ -205,7 +205,7 @@ void WPSGraphicStyle::addTo(librevenge::RVNGPropertyList &list, bool only1D) con
 			list.insert("draw:dots2", nDots2);
 			list.insert("draw:dots2-length", double(size2), librevenge::RVNG_POINT);
 		}
-		const double distance = double(nDots1 + nDots2 > 0 ? totalGap/float(nDots1+nDots2) : totalGap);
+		const auto distance = double(nDots1 + nDots2 > 0 ? totalGap/float(nDots1+nDots2) : totalGap);
 		list.insert("draw:distance", distance, librevenge::RVNG_POINT);;
 	}
 	else
